#include "stdafx.h"
#include "ClippingFilterDlg.h"
#include "ClippingFilter.h"

//#include "ClippingFilterres.h"
#include "resource.h"

// Ignore some stuff from commctrl.h
#define    NOTOOLBAR    Customizable bitmap-button toolbar control.
#define    NOUPDOWN     Up and Down arrow increment/decrement control.
#define    NOSTATUSBAR  Status bar control.
#define    NOMENUHELP   APIs to help manage menus, especially with a status bar.
//#define    NOTRACKBAR   Customizable column-width tracking control.
#define    NODRAGLIST   APIs to make a listbox source and sink drag&drop actions.
#define    NOPROGRESS   Progress gas gauge.
#define    NOHOTKEY     HotKey control
#define    NOHEADER     Header bar control.
#define    NOIMAGEAPIS  ImageList apis.
#define    NOLISTVIEW   ListView control.
#define    NOTREEVIEW   TreeView control.
#define    NOTABCONTROL Tab control.
#define    NOANIMATE    Animate control

#include <commctrl.h>
#include <commdlg.h>

#ifdef FILTERDLL_EXPORTS
extern HANDLE g_hModule;
#endif


//////////////////////////////////////////////////////////////////////////
// Dialog Message Handler
//////////////////////////////////////////////////////////////////////////

LRESULT CALLBACK CClippingFilterDlgMessage( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam )
{
	if( message == WM_INITDIALOG )
	{
		::SetWindowLong( hDlg,  (-21), (long)lParam );
	}
	((CClippingFilterDlg*)::GetWindowLong( hDlg,  (-21) ) )->DlgMessage( hDlg, message, wParam, lParam );

	return FALSE; 
}


//////////////////////////////////////////////////////////////////////////

CClippingFilterDlg::CClippingFilterDlg( CClippingFilter* pCClippingFilter )
{
	m_pCClippingFilter = pCClippingFilter;
}

void CClippingFilterDlg::show(HWND hParent)
{ 	
#ifdef FILTERDLL_EXPORTS
	DialogBoxParam( (HINSTANCE)g_hModule, (LPCTSTR)IDD_DIALOG_CCLIPPINGFILTER, hParent, 
		(DLGPROC)CClippingFilterDlgMessage, (long)this );
#else
	DialogBoxParam( AfxGetInstanceHandle(), (LPCTSTR)IDD_DIALOG_CCLIPPINGFILTER, hParent, 
		(DLGPROC)CClippingFilterDlgMessage, (long)this );
#endif
}


//////////////////////////////////////////////////////////////////////////
// The message handler for the dialog
//////////////////////////////////////////////////////////////////////////

LRESULT CClippingFilterDlg::DlgMessage( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam )
{
	switch (message)
	{
		case WM_INITDIALOG:
			CreateControls( hDlg );
			return TRUE;

		case WM_HSCROLL:
			OnSliderScroll( hDlg );
			break;

		case WM_PAINT:
			OnPaint( hDlg );
			break;

		case WM_COMMAND:
			switch( LOWORD( wParam ) )
			{
				case IDC_CHECKACTIVE:
					OnCheckEnable( hDlg );
					break;

				case IDC_PATTERN_RADIO:
				case IDC_COLOR_RADIO:
				case IDC_CLIP_ABOVE_RADIO:
				case IDC_CLIP_BELOW_RADIO:
					OnCheckRadioButton( hDlg, LOWORD(wParam) );
					break;				

				case IDC_SELECTCOLOR_BUTTON:
					OnSelectColor( hDlg );
					break;

				case IDOK:
					::EndDialog( hDlg, LOWORD(wParam) );
					return TRUE;
			}
			break;
	}
	return FALSE;
}


//////////////////////////////////////////////////////////////////////////
// Create the controls of the dialog and initialize them according
// to the appropriate parameter values.
//////////////////////////////////////////////////////////////////////////

void CClippingFilterDlg::CreateControls( HWND hDlg )
{
	bool bEnabled;
	m_pCClippingFilter->getEnabled( bEnabled );
	if( bEnabled )
	{
		::SendDlgItemMessage( hDlg, IDC_CHECKACTIVE, BM_SETCHECK, BST_CHECKED, 0 );
	}
	else
	{
		::SendDlgItemMessage( hDlg, IDC_CHECKACTIVE, BM_SETCHECK, BST_UNCHECKED, 0 );
	}

	long threshold;
	m_pCClippingFilter->getThreshold( threshold );

	::SendDlgItemMessage( hDlg, IDC_THRESHOLD_SLIDER, TBM_SETRANGE, FALSE, MAKELONG(0,255) );
	::SendDlgItemMessage( hDlg, IDC_THRESHOLD_SLIDER, TBM_SETPOS, TRUE, threshold );
	::SetDlgItemInt( hDlg, IDC_THRESHOLD_STATIC, threshold, FALSE );

	bool bClipAboveThreshold;
	m_pCClippingFilter->getClipAboveThreshold( bClipAboveThreshold );
	CheckRadioButton(hDlg, IDC_CLIP_ABOVE_RADIO, IDC_CLIP_BELOW_RADIO,
		(bClipAboveThreshold) ? IDC_CLIP_ABOVE_RADIO : IDC_CLIP_BELOW_RADIO);

	long fillmode;
	m_pCClippingFilter->getFillMode( fillmode );
	::CheckRadioButton( hDlg, IDC_PATTERN_RADIO, IDC_COLOR_RADIO, IDC_PATTERN_RADIO + fillmode );
	::EnableWindow( ::GetDlgItem( hDlg, IDC_SELECTCOLOR_BUTTON ), (fillmode == 0) ? FALSE : TRUE );

	COLORREF fillcolor;
	m_pCClippingFilter->getFillColor( (long&)fillcolor );

	::ShowWindow( hDlg, SW_SHOW );
}


//////////////////////////////////////////////////////////////////////////
// Handle the Enabled check box.
//////////////////////////////////////////////////////////////////////////

void CClippingFilterDlg::OnCheckEnable( HWND hDlg )
{
	if( ::SendDlgItemMessage( hDlg, IDC_CHECKACTIVE, BM_GETCHECK, 0, 0 ) == BST_CHECKED )
	{
		m_pCClippingFilter->setEnabled( true );
	}
	else
	{
		m_pCClippingFilter->setEnabled( false );
	}
}

void CClippingFilterDlg::OnSliderScroll( HWND hDlg )
{
	int pos = (int)( ::SendDlgItemMessage( hDlg, IDC_THRESHOLD_SLIDER, TBM_GETPOS, 0, 0 ));

	m_pCClippingFilter->setThreshold( pos );
	::SetDlgItemInt( hDlg, IDC_THRESHOLD_STATIC, pos, FALSE );
}

void CClippingFilterDlg::OnCheckRadioButton( HWND hDlg, UINT id )
{
	switch( id )
	{
	case IDC_PATTERN_RADIO:
		m_pCClippingFilter->setFillMode( 0 );
		::CheckRadioButton( hDlg, IDC_PATTERN_RADIO, IDC_COLOR_RADIO, IDC_PATTERN_RADIO);
		::EnableWindow( ::GetDlgItem( hDlg, IDC_SELECTCOLOR_BUTTON ), FALSE );
		break;
	case IDC_COLOR_RADIO:
		m_pCClippingFilter->setFillMode( 1 );
		::CheckRadioButton( hDlg, IDC_PATTERN_RADIO, IDC_COLOR_RADIO, IDC_COLOR_RADIO );
		::EnableWindow( ::GetDlgItem( hDlg, IDC_SELECTCOLOR_BUTTON ), TRUE );
		break;
	case IDC_CLIP_BELOW_RADIO:
		m_pCClippingFilter->setClipAboveThreshold(false);
		CheckRadioButton(hDlg, IDC_CLIP_ABOVE_RADIO, IDC_CLIP_BELOW_RADIO, IDC_CLIP_BELOW_RADIO);
		break;
	case IDC_CLIP_ABOVE_RADIO:
		m_pCClippingFilter->setClipAboveThreshold(true);
		CheckRadioButton(hDlg, IDC_CLIP_ABOVE_RADIO, IDC_CLIP_BELOW_RADIO, IDC_CLIP_ABOVE_RADIO);
		break;
	}
}

/*
 * OnSelectColor
 * Use the Win32 ChooseColor function to let the user select a fill color
 */
void CClippingFilterDlg::OnSelectColor( HWND hDlg )
{
	static COLORREF customColors[16]; 

	// Fill the CHOOSECOLOR structure
	CHOOSECOLOR chooseColor;
	ZeroMemory( &chooseColor, sizeof(chooseColor) );
	chooseColor.lStructSize = sizeof(chooseColor);
	chooseColor.hwndOwner = hDlg;
	m_pCClippingFilter->getFillColor( (long&)chooseColor.rgbResult );
	chooseColor.Flags = CC_RGBINIT;
	chooseColor.lpCustColors = customColors;

	// Call ChooseColor
	if( ::ChooseColor( &chooseColor ) )
	{
		// If the user clicked 'OK' set the new color
		m_pCClippingFilter->setFillColor( (long)chooseColor.rgbResult );
		::InvalidateRect( hDlg, 0, FALSE );
	}
}

/*
 * OnPaint
 * Paint a colored rectangle indicating the currently selected fill color next to the
 * color selection button
 */
void CClippingFilterDlg::OnPaint( HWND hDlg )
{
	HWND hButton = ::GetDlgItem( hDlg, IDC_SELECTCOLOR_BUTTON );

	// Get the button position and map it to dialog coordinates
	RECT buttonRect;
	::GetWindowRect( hButton, &buttonRect );
	::ScreenToClient( hDlg, (LPPOINT)&buttonRect);
	::ScreenToClient( hDlg, ((LPPOINT)&buttonRect)+1);

	// Calculate the position of the rectangle next to the button
	int h = buttonRect.bottom - buttonRect.top;
	RECT colorRect;
	colorRect.left = buttonRect.right + h/2;
	colorRect.right = colorRect.left + h;
	colorRect.top = buttonRect.top;
	colorRect.bottom = buttonRect.bottom;

	// Get the current fill color from the frame filter
	COLORREF currentColor;
	m_pCClippingFilter->getFillColor( (long&)currentColor );

	// Paint the rectangle
	PAINTSTRUCT ps;
	HDC hDC = ::BeginPaint( hDlg, &ps );

	HBRUSH hColorBrush = ::CreateSolidBrush( currentColor );

	::FillRect( hDC, &colorRect, hColorBrush );
	::FrameRect( hDC, &colorRect, (HBRUSH)GetStockObject( BLACK_BRUSH ) );

	DeleteObject( hColorBrush );

	::EndPaint( hDlg, &ps );
}
