#include "stdafx.h"
#include "ContrastEnhancement.h"
#include "ContrastEnhancementDlg.h"

using namespace _DSHOWLIB_NAMESPACE;


CContrastEnhancement::CContrastEnhancement()
{
	m_bEnabled = true;

	// Default values for the lower and upper bound
	m_lowerBound = 50;
	m_upperBound = 255 - 50;

	// The parameter input and output methods must be members of the class
	addLongParam( L"Lower Bound", &CContrastEnhancement::getLowerBound, &CContrastEnhancement::setLowerBound );
	addLongParam( L"Upper Bound", &CContrastEnhancement::getUpperBound, &CContrastEnhancement::setUpperBound );
	addBoolParam( L"Enable", &CContrastEnhancement::getEnabled, &CContrastEnhancement::setEnabled );
}

//////////////////////////////////////////////////////////////////////////

_DSHOWLIB_NAMESPACE::FilterInfo CContrastEnhancement::getStaticFilterInfo()
{
	static _DSHOWLIB_NAMESPACE::FilterInfo f = { L"ContrastEnhancement", L"", eFC_GENERIC };
	return f;

}
//////////////////////////////////////////////////////////////////////////
// Implementation of the parameter input and output methods.
/////////////////////////////////////////////////////////////////////////

tFilterError CContrastEnhancement::setEnabled( bool bVal )
{
	m_bEnabled = bVal;
	return eNO_ERROR;
}
tFilterError CContrastEnhancement::getEnabled( bool &bVal )
{
	bVal = m_bEnabled;
	return eNO_ERROR;
}

tFilterError CContrastEnhancement::setLowerBound( long lVal )
{
	// The value must be in a range between 0 and m_upperBound
	if( lVal >= m_upperBound || lVal < 0 )
	{
		return eINVALID_PARAM_VALUE;
	}

	m_lowerBound = lVal;
	return eNO_ERROR;
}

tFilterError CContrastEnhancement::getLowerBound( long &lVal )
{
	lVal = m_lowerBound;
	return eNO_ERROR;
}

tFilterError CContrastEnhancement::setUpperBound( long lVal )
{
	// The value must be in a range between m_upperBound and 255
	if( lVal <= m_lowerBound || lVal > 255 )
	{
		return eINVALID_PARAM_VALUE;
	}

	m_upperBound = lVal;
	return eNO_ERROR;
}

tFilterError CContrastEnhancement::getUpperBound( long &lVal )
{
	lVal = m_upperBound;
	return eNO_ERROR;
}

//////////////////////////////////////////////////////////////////////////
// This method is used by the framework to check whether this filter has a
// property dialog.
//////////////////////////////////////////////////////////////////////////

bool CContrastEnhancement::hasDialog() const
{
	return true;	// This class has its own property dialog.
}
//////////////////////////////////////////////////////////////////////////
// Show the dialog. This method is called by the framework.
//////////////////////////////////////////////////////////////////////////

bool CContrastEnhancement::callDialog( HWND hParent )
{
	CContrastEnhancementDlg dlg( this );
	dlg.show( hParent );

	return true;
}


//////////////////////////////////////////////////////////////////////////
// The array "arr" is filled with the color formats that are supported by
// this filter. 
//////////////////////////////////////////////////////////////////////////

void CContrastEnhancement::getSupportedInputTypes( FrameTypeInfoArray& arr ) const 
{
	arr.addFrameType( eRGB8 );
	arr.addFrameType( eY800 );
	arr.addFrameType( eRGB24 );
	arr.addFrameType( eRGB32 );
}
//////////////////////////////////////////////////////////////////////////

bool CContrastEnhancement::getTransformOutputTypes( const FrameTypeInfo& in_type, FrameTypeInfoArray& out_types ) const 
{
	// This filter does not do any color space conversion. Therefore, the input format is the output format.
	out_types.push_back( in_type );
	return true;
};

//////////////////////////////////////////////////////////////////////////
// This is the transformation method. If m_bEnabled is true, every pixel 
// value is mapped to a range between m_lowerBound and m_upperBound.
// If m_bEnabled is false, the source frame is copied to the
// destination frame without any changes.
//////////////////////////////////////////////////////////////////////////

bool CContrastEnhancement::transform( const IFrame& src, IFrame& dest )
{
	// If the filter is used in a FrameHandlerSink, and no MemBufferCollection
	// is specified or all MemBuffers are locked, the destination frame pointer is 0.
	if( dest.getPtr() == 0 ) return false;

	BYTE LUT[256];

	// Initialize a lookup table with values between m_lowerBound and m_upperBound
	for( int i = 0; i < 256; ++i )
	{
		if( i <= m_lowerBound )
		{
			LUT[i] = 0;
		}
		else if( i >= m_upperBound )
		{
			LUT[i] = 255;
		}
		else
		{
			LUT[i] = (BYTE)(float(i - m_lowerBound) / float(m_upperBound - m_lowerBound) * 255.0f);
		}
	}

	if( !m_bEnabled )
	{
		// If the filter is not enabled, copy all pixels without any changes.
		memcpy( dest.getPtr(),src.getPtr(), dest.getFrameType().buffersize );
	}
	else
	{
		// Map all pixel values to a range defined by the lookup table.
		BYTE* pSource = src.getPtr();
		BYTE* pTarget = dest.getPtr();
		BYTE *pEnd =  &pSource[src.getFrameType().buffersize];

		while( pSource < pEnd )
		{
			*pTarget = LUT[*pSource];
			pSource++;
			pTarget++;
		}
	}
	return true;
}
